program scorch2d;

{$IFDEF Win32}
  {$IFDEF FPC}
    {$APPTYPE GUI}     // FreePascal/Lazarus
  {$ELSE}
    {$APPTYPE CONSOLE} // Delphi
  {$ENDIF}
{$ENDIF}

uses
  SysUtils,
  // JEDI-SDL
  sdl,
  sdlutils,

  GraphicsUnit,
  GameObjectUnit,
  GameConstantsUnit;

type
  // Game Modes
  TGameMode = (gmMainMenu, gmAiming, gmShooting, gmMenu, gmQuit);

  // Game Controls
  TGameControls = Record
    Up, Down, Left, Right: Boolean;
    Fire, Select, Menu: Boolean;
  end;
  
var
  GAME_FPS: Cardinal = 30; {30}

  // Video Screens
  GameScreen: PSDL_Surface; // Main GameScreen!

  // Game Data
  GameMode: TGameMode = gmAiming;
  RunClock: Integer;

  GameInput: TGameControls;
  
  // Level Data
  Level: TBattlefield;

  // Tank Data
  NumberOfTanks: Integer;
  Tanks: Array[0 .. 3] of TTank;

function Get_FPS(fps: Cardinal): UInt32;
begin
     if (fps > 0) then
        Result := 1000 div fps
     else
         Result := 0;
end;


procedure doGameInput;
var
  event: TSDL_Event;
begin
     while (SDL_PollEvent(@event) > 0) do
     begin
          case (event.type_) of
            SDL_KEYDOWN : begin
                               if (event.key.keysym.sym = SDLK_UP) then
                                  GameInput.Up := True;
                               if (event.key.keysym.sym = SDLK_DOWN) then
                                  GameInput.Down := True;
                               if (event.key.keysym.sym = SDLK_LEFT) then
                                  GameInput.Left := True;
                               if (event.key.keysym.sym = SDLK_RIGHT) then
                                  GameInput.Right := True;
                               if (event.key.keysym.sym = SDLK_SPACE) or (event.key.keysym.sym = SDLK_RETURN) then
                                  GameInput.Fire := True;
                               if (event.key.keysym.sym = SDLK_TAB) then
                                  GameInput.Select := True;
                               if (event.key.keysym.sym = SDLK_ESCAPE) then
                                  GameInput.Menu := True;
                          end;
            SDL_KEYUP   : begin
                               if (event.key.keysym.sym = SDLK_UP) then
                                  GameInput.Up := False;
                               if (event.key.keysym.sym = SDLK_DOWN) then
                                  GameInput.Down := False;
                               if (event.key.keysym.sym = SDLK_LEFT) then
                                  GameInput.Left := False;
                               if (event.key.keysym.sym = SDLK_RIGHT) then
                                  GameInput.Right := False;
                               if (event.key.keysym.sym = SDLK_SPACE) or (event.key.keysym.sym = SDLK_RETURN) then
                                  GameInput.Fire := False;
                               if (event.key.keysym.sym = SDLK_TAB) then
                                  GameInput.Select := False;
                               if (event.key.keysym.sym = SDLK_ESCAPE) then
                                  GameInput.Menu := False;
                          end;
          end;
     end;
     
     // -- Game Key Actions -- //
     if (GameInput.Menu) then
        GameMode := gmQuit;

     // -- Reset Game Keys -- //
     GameInput.Fire := False;
     GameInput.Select := False;
     GameInput.Menu := False;
end;

procedure GameCycle;
begin
end;

procedure DrawScreen;
var
  i: Integer;
begin
     Level.DrawSky(GameScreen);
     Level.DrawLand(GameScreen);

     for i := 0 to NumberOfTanks - 1 do
         Tanks[i].Draw(GameScreen);

     SDL_Flip(GameScreen);
end;

// -- INIT & DEINIT -- //
procedure ProgramCreate;
var
  Buffer, s: Word;
begin
     // Update Random Seed value (Helps make random values more random!)
     Randomize;
     DecodeTime(Time, Buffer, Buffer, Buffer, s);
     RandSeed := s;

     // Init Video Mode //
     if (SDL_Init(SDL_INIT_VIDEO) <> 0) then
        Halt;

     // Set the title bar in environments that support it
     SDL_WM_SetCaption('Scorch 2D', nil);

     // Set Video Mode
     GameScreen := SDL_SetVideoMode(800, 600, 32, SDL_SWSURFACE);
     if (GameScreen = nil) then
     begin
          SDL_Quit;
          Halt;
     end;

     // Hide Cursor
     SDL_ShowCursor(SDL_DISABLE);
end;
// Deinit Program
procedure ProgramClose;
begin
     // Show Cursor
     SDL_ShowCursor(SDL_ENABLE);

     // Shutdown SDL
     SDL_Quit;
end;
// -- End of INIT & DEINIT -- //


 // ~~~ | ------------ | ~~~ //
 // === | MAIN PROGRAM | === //
 // ~~~ | ------------ | ~~~ //
begin
     ProgramCreate;

     // Create Battlefield
     Level := TBattlefield.Init(GameScreen.w, GameScreen.h, $229900, $0000ee, True, 'DesertEclipse.bmp');
     Level.GenerateLand(LandHighest, LandLowest, LandVariation);
     Level.SmoothenLand(LandSmoothing);

     // Place Tanks
     NumberOfTanks := 4;
     Tanks[0] := TTank.Init(13, -1, -8, 12, $008442, 'TK-1.bmp');
     Tanks[1] := TTank.Init(13, 0, -8, 12, $e74218, 'TK-2.bmp');
     Tanks[2] := TTank.Init(13, -4, -8, 13, $8400ff, 'TK-3.bmp');
     Tanks[3] := TTank.Init(13, -5, -8, 14, $ffff84, 'TK-4.bmp');

     Level.ScatterTanks(Tanks, 4, TankMinDistance, TankPlaceGap);

     Level.SmoothenLand(1);

     RunClock := 0;
     repeat
       SDL_Delay(Get_FPS(GAME_FPS));

       doGameInput;

       GameCycle;

       DrawScreen;

       inc(RunClock);
     until (GameMode = gmQuit); // Exit when any key is pressed!

     Tanks[0].Free;
     
     Level.Free;
     
     ProgramClose;
end.
